package Test::MyCmd::Command::stock;

use strict;
use warnings;

use parent qw(App::Cmd::Command);

=head1 NAME

Test::MyCmd::Command::stock - nothing here is overridden

=cut

# This package exists to test all the default command plugin behaviors.

1;
